#ifndef INPUT_H
#define INPUT_H

#include <cstdlib>
#include <vector>
#include <set>
#include <iostream>
#include <string>


using namespace std;

// Input class

class Graph
  {
    friend std::ostream& operator<<(std::ostream&, const Graph&);
  public:
    Graph();
    /* this method loads a DIMACS coded graph coloring instance */
    void Load(const char *id, bool print_statistics);  
    unsigned int Vertices() const  { return num_of_vertices; }
    unsigned int Edges() const  { return num_of_edges; }
    virtual ~Graph();
    bool Edge(const int& v, const int& w) const
      { return adjacency_matrix[v][w] != 0; }
    float Density() const {return density;}
    float AvDegree() const {return av_degree;}
    
    vector< set<int> > adjacency_set;
    vector<unsigned int> degree;
    vector<pair<int, int> > edges;
  protected:
    int num_of_vertices;
    int num_of_edges;
    int max_degree;
    int min_degree;

    int declared_edges;

    float density;
    float av_degree;
    unsigned char **adjacency_matrix;       
  private:
    void read_DIMACS_line(char *buf);
 };


#endif



