#include "Random.h"

/* Random Generator */
double Random::MRG32k3a()
{
   long    k;
   double p1, p2;
   /* Component 1 */
   p1 = a12 * s11 - a13n * s10;
   k = (long) (p1 / m1);
   p1 -= k * m1;  
   if (p1 < 0.0) p1 += m1;
   s10 = s11; s11 = s12; s12 = p1;
   /* Component 2 */
   p2 = a21 * s22 - a23n * s20;
   k = (long) (p2 / m2); p2 -= k * m2;
   if (p2 < 0.0) p2 += m2;
   s20 = s21; s21 = s22; s22 = p2;
   /* Combination */
   if (p1 <= p2) return ((p1 - p2 + m1) * norm);
   else return ((p1 - p2) * norm);
}


double Random::ranU01()
{
   return MRG32k3a();
}

/* Uniform random int in [i,j] */
int Random::ranUint(int i, int j)
{
   return i + (int) ((j - i + 1.0) * MRG32k3a());
}




long int* Random::generate_array(const int& size) {
   long int  i, j, help;
   long int  *v;

   v = new long int[size];

   for ( i = 0 ; i < size; i++ ) 
     v[i] = i;

   for ( i = 0 ; i < size-1 ; i++) {
     j = (long int) ( ranU01( ) * (size - i)); 
     VERYVERBOSE(assert( i + j < size );)
     help = v[i];
     v[i] = v[i+j];
     v[i+j] = help;
   }
   VERYVERBOSE ( printf("Random vector:\n");
   for (i = 0 ; i < size ; i++ ) 
     printf(" %ld ",v[i]);
   printf("\n"); )
   return v;
}

